#!/usr/bin/env python3
import os,sys,traceback

#############################################################
logfile = "/var/log/kodak.log"
http_port = 80
modules = {
################################################################
#  Nom du module  # Position sur l'écran # Refresh en secondes #
################################################################
   'background' : {'x' :   0, 'y' :   0, 'refresh' : -1   },   #
   'chloe'      : {'x' : 690, 'y' : 290, 'refresh' : -1   },   #
   'date_fete'  : {'x' :  30, 'y' : 287, 'refresh' : 3600 },   #
   'heure'      : {'x' :  30, 'y' : 262, 'refresh' : -1   },   #
   'meteo'      : {'x' : 350, 'y' :  20, 'refresh' : 1800 },   #
   'cts'        : {'x' :  30, 'y' : 340, 'refresh' : -1   },   #
################################################################
}

modules_list = []
modules_call = None
f = None

#Http server
from http.server import BaseHTTPRequestHandler, HTTPServer
from time import strftime, gmtime, time
import select
last_fetch = 0
next_time = 0
last_path = "/"
last_request = ""
do_timeout = 0
httpd = None

#module background
#apt-get install python3-cairo
#apt-get install python3-pip libjpeg-dev
#pip-3.2 install Pillow
import cairo
from io import BytesIO
from PIL import Image
screen_jpg = BytesIO()

#Logging
import log
from sys import stdout
import signal

def load_binary(file):
   with open(file, 'rb') as file:
      return file.read()

#################################################################
# BUILD SCREEN                                                  #
#################################################################
def build_screen():
   ctx = None
   surface = None
   max_modules = len(modules_list)
   log.write("Building Screen...")
   chrono = int(round(time() * 1000))
   chrono_total = chrono
   j = 2

   surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 800, 480)
   ctx = cairo.Context (surface)

## Chargement du background en premier  
   for i in range(0,max_modules):
      if modules_list[i] == 'background':
         try:
            modules_call[i].draw(ctx, modules[modules_list[i]]['x'], modules[modules_list[i]]['y'])
         except Exception:
            tb = traceback.format_exc()
            log.write(tb)            
         log.write('\t1/' + str(max_modules + 3) + ' Module ' + modules_list[i] + "\t\t" + str(int(round(time() * 1000)) - chrono) + "ms")
         chrono = int(round(time() * 1000))
## Chargement des autres modules
   for i in range(0,max_modules):
      if modules_list[i] != 'background' and modules_list[i] != 'chloe':
         try:
            modules_call[i].draw(ctx, modules[modules_list[i]]['x'], modules[modules_list[i]]['y'])
         except Exception:
            tb = traceback.format_exc()
            log.write(tb)            
         log.write("\t" + str(j) + '/' + str(max_modules + 3) + ' Module ' + ("%10s" % modules_list[i]) + "\t\t" + str(int(round(time() * 1000)) - chrono) + "ms")
         chrono = int(round(time() * 1000))
         j = j + 1
## Chargement du module chloe en dernier
   for i in range(0,max_modules):
      if modules_list[i] == 'chloe':
         try:
            modules_call[i].draw(ctx, modules[modules_list[i]]['x'], modules[modules_list[i]]['y'])
         except Exception:
            tb = traceback.format_exc()
            log.write(tb)            
         log.write("\t" + str(j) + '/' + str(max_modules + 3) + ' Module ' + ("%10s" % modules_list[i]) + "\t\t" + str(int(round(time() * 1000)) - chrono) + "ms")
         chrono = int(round(time() * 1000))

## Ecrire le JPG
   png = BytesIO()
   surface.write_to_png(png)
   log.write("\t" + str(max_modules + 1) + "/" + str(max_modules + 3) + " Write to PNG \t\t" + str(int(round(time() * 1000)) - chrono) + "ms")
   chrono = int(round(time() * 1000))
   im = Image.open(png)
   log.write("\t" + str(max_modules + 2) + "/" + str(max_modules + 3) + " Init conv to JPG \t\t" + str(int(round(time() * 1000)) - chrono) + "ms")
   chrono = int(round(time() * 1000))
   screen_jpg.seek(0)
   im.save(screen_jpg, 'JPEG', quality=100)
   png.close()
   im.close()
   log.write("\t" + str(max_modules + 3) + "/" + str(max_modules + 3) + " Converting to JPG \t\t" + str(int(round(time() * 1000)) - chrono) + "ms")
   log.write("\tTOTAL :  \t\t\t" + str(int(round(time() * 1000)) - chrono_total) + "ms")

#################################################################
# HTTPRequestHandler class                                      #
#################################################################
class testHTTPServer_RequestHandler(BaseHTTPRequestHandler):
   def do_GET(self):
      global last_path, last_fetch, do_timeout, last_request
      self.close_connection = 1
      message = ""
      content_type = "text/html"
      binary = 0
      
      last_request = ("%s -> %s - \"%s %s %s\"" % (self.headers.get('Host'), self.client_address[0], self.command, self.request_version, self.path))

      if self.path == "/favicon.ico":
         self.send_response(404)
         return
      elif self.path == "/":
         content_type = "application/rss+xml"
         timestamp = str(int(time()))
         message = "<?xml version='1.0' encoding='utf-8'?>\n<rss version='2.0' xmlns:media='http://search.yahoo.com/mrss/'>\n\t<channel>\n\t\t<title>Chloe RSS</title>\n\t\t<ttl>1</ttl>"
         for i in range(0, 4):
            timestamp = str(int(time()) + (i * 5))
            message += "\n\t\t<item>"
            message += "\n\t\t\t<pubDate>" + timestamp + "</pubDate>" 
            message += "\n\t\t\t<guid isPermaLink='false'>guid-" + timestamp + "</guid>"
            message += "\n\t\t\t<media:content url='http://" + self.headers.get('Host') + "/screen" + str(i) + "?date=" + timestamp + "' type='image/jpeg' />"
            message += "\n\t\t</item>"
         message += "\n\t</channel>\n</rss>"
      elif self.path.startswith("/screen"):
         if last_path != self.path:
            last_path = self.path
            do_timeout = 1
         content_type = "image/jpeg"
         message = screen_jpg.getvalue()
         binary = 1
 
      if binary == 0:
         message = bytes(message, "utf8")

      # Send response status code
      self.send_response(200)
      self.send_header('Content-type',content_type)
      self.send_header('Content-Length',len(message))
      self.send_header('Cache-Control','no-store, no-cache, must-revalidate, pre-check=0, post-check=0, max-age=0')
      self.send_header('Pragma','no-cache')
      self.send_header('Expires',strftime("%a, %d %b %Y %H:%M:%S +0000", gmtime(int(time()+15))))
      self.send_header('Last-Modified',strftime("%a, %d %b %Y %H:%M:%S +0000", gmtime(int(time()-10))))
      self.end_headers()
      self.wfile.write(message)
      return 

   def log_message(self, format, *args):
      return

def signal_term_handler(signal, frame):
   global httpd, f
   print('SIGTERM signal')
   log.write('Shutting down the web server')
   httpd.socket.close()
   f.close()
   sys.exit(0)   
#################################################################
# main program                                                  #
#################################################################
try:
   build_screen_time = 0
   f = open(logfile, 'a')
   log.init(f)
   signal.signal(signal.SIGTERM, signal_term_handler)
   root_path = os.path.dirname(os.path.realpath(__file__))
 
##### Initialisation des modules
   log.write('Initializing modules...')
   for name, parameter in modules.items():
      modules_list.append(name)
   modules_call = list(map(__import__, modules_list))
   for i in range(0,len(modules_list)):
      log.write('\t- Loading Module ' + modules_list[i] + '...', end="")
      modules_call[i].init(root_path)
      log.cat('\n',end="")

##### Calcul du premier écran
   for i in range(0,len(modules_list)):
      if modules[modules_list[i]]['refresh'] >= 0:
         try:
            modules_call[i].refresh(modules[modules_list[i]]['refresh']) 
         except Exception:
            tb = traceback.format_exc()
            log.write(tb)
   time_1 = time()
   build_screen()
   time_2 = time() - time_1
   if(time_2 > build_screen_time) :
      build_screen_time = int(time_2)

##### Démarrage du serveur HTTP
   log.write('Starting HTTP server...',end="")
   server_address = ('', http_port)
   testHTTPServer_RequestHandler.protocol_version = "HTTP/1.1"
   httpd = HTTPServer(server_address, testHTTPServer_RequestHandler)
   log.cat('port ' + str(http_port) + '!')
   inputs = [httpd.fileno()]
   while 1:
##### Refresh des modules
      for i in range(0,len(modules_list)):
         if modules[modules_list[i]]['refresh'] >= 0:
            try:
               modules_call[i].refresh(modules[modules_list[i]]['refresh'])
            except Exception:
               tb = traceback.format_exc()
               log.write(tb)

##### Calcul du timeout select
      timeout = int(next_time) - (build_screen_time*2)
      if timeout <= 0:
         log.write("Select without timeout")
         readable, writable, exceptional = select.select(inputs, [], inputs)   
      else:
         timeout = 20
         log.write("Select timeout in " + str(timeout) + "s")
         readable, writable, exceptional = select.select(inputs, [], inputs, timeout)   
##### Preparation de l'écran
      if len(readable) == 0:
         log.write("TIMEOUT!")
         time_1 = time()
         build_screen()
         time_2 = time() - time_1
         log.write("Build script time : " + str(time_2) + "s")
         if(time_2 > build_screen_time) :
            build_screen_time = int(time_2)
         next_time = 0
      else:
##### Processing de la requete HTTP
         for s in readable:
            do_timeout = 0
            httpd.handle_request()
            log.write(last_request)
            if do_timeout == 1:
               next_time = int(time()) - last_fetch
               last_fetch = int(time())
               log.write("Delai avant prochaine image : " + str(next_time))
         #      build_screen()
except KeyboardInterrupt:
   log.write('^C SIGINT received')
except Exception:
   tb = traceback.format_exc()
   log.write(tb)
finally:
   log.write('Shutting down the web server')
   httpd.socket.close()
   f.close()
