#!/usr/bin/env python3

import log
from math import pi
import cairo
import pycurl
from io import BytesIO
from xml.etree import ElementTree
from time import localtime, strftime, time
import os
import matplotlib
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d
import numpy as np
import locale

#############################################################
#Renseigner la clé de l'API wunderground.com (créer un compte)
api = ''
#Ville de référence pour les prévisions
city = 'FR/Strasbourg'
#############################################################

nb_fcast = 5
nb_temperature = 12
icons = {}
width = 400
height = 440
radius = 10

meteo_surface = None
last_update = 0

old_temperature = ""
old_today = ""
old_previsions = ""

def textwidth(ctx,text, fontsize):
   ctx.set_font_size(fontsize)
   xbearing, ybearing, width, height, xadvance, yadvance = ctx.text_extents(text)
   return width

def middle(ctx,text,fontsize,total):
   return (total - textwidth(ctx,text, fontsize)) /2

def download(url):
   response = ""
   try:
      buffer = BytesIO()
      c = pycurl.Curl()
      c.setopt(c.URL, url)
      c.setopt(c.CONNECTTIMEOUT, 5);
      c.setopt(c.TIMEOUT, 5);
      c.setopt(c.FOLLOWLOCATION, True)
      c.setopt(c.WRITEDATA, buffer)
      c.perform()
      c.close()
      response = buffer.getvalue() #.decode('iso-8859-1')
   except Exception:
      response = ""
   finally:
      return response

def refresh(cron):
   global meteo_surface, last_update, old_temperature, old_previsions, old_today
   if int(time()) < (last_update + cron):
      return
   log.write("Refreshing meteo...")
   last_update = time()
   meteo_surface.finish()
   meteo_surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
   ctx = cairo.Context(meteo_surface)
   ctx.select_font_face('Droid Sans')
   ctx.set_source_rgba(0, 0, 0,0.5)
   ctx.arc(radius, radius, radius, 2*(pi/2), 3*(pi/2))
   ctx.arc(width - radius, radius, radius, 3*(pi/2), 4*(pi/2))
   ctx.arc(width - radius, height - radius, radius, 0*(pi/2), 1*(pi/2))  # ;o)
   ctx.arc(radius, height - radius, radius, 1*(pi/2), 2*(pi/2))
   ctx.close_path()
   ctx.fill()
##############################################################
   result = ""
#   result = meteo_debug.download_10days()
   result = download('http://api.wunderground.com/api/' + api + '/forecast10day/lang:FR/q/' + city + '.xml')
   if len(result) > 0 :
      validate = 0
      xml = ElementTree.fromstring(result)
      for root in xml.findall('forecast'):
         for txt in root.findall('simpleforecast'):
            for days in txt.findall('forecastdays'):
               for day in days.findall('forecastday'):
                  validate = 1
                  old_previsions = result
      if validate == 0:
         result = ""
   if len(result) < 1 :
      result = old_previsions 
   if len(result) > 0 :
      pop = previsions(result,ctx) 
##############################################################
   result = ""
#   result = meteo_debug.download_today()
   result = download('http://api.wunderground.com/api/' + api + '/conditions/lang:FR/q/' + city + '.xml')
   if len(result) > 0 :
      validate = 0
      xml = ElementTree.fromstring(result)
      if len(xml.findtext(".//temp_c")) > 0:
         if len(xml.findtext(".//feelslike_c")) > 0:
            if len(xml.findtext(".//icon")):
               if len(xml.findtext(".//relative_humidity")):
                  if len(xml.findtext(".//wind_kph")):
                     if len(xml.findtext(".//observation_epoch")):
                        validate = 1
                        old_today = result
      if validate == 0:
         result = ""
   if len(result) < 1 :
      result = old_today
   if len(result) > 0 :
      today(result,ctx,pop)
##############################################################  
   result = ""
   labels = []
   hours = []
   temp_feelslike = []
   rainchance = []
   xml_hour2 = 0
   xml_day2 = 0
   i = 0
#   result = meteo_debug.download_temperature()
   result = download('http://api.wunderground.com/api/' + api + '/hourly/lang:FR/q/' + city + '.xml')
   if len(result) > 0 :
      xml = ElementTree.fromstring(result)
      for root in xml.findall('hourly_forecast'):
         for forecast in root.findall('forecast'):
            for fcttime in forecast.findall('FCTTIME'):
               xml_hour = int(fcttime.find('hour_padded').text)
               if xml_hour2 > xml_hour:
                  xml_day2 = xml_day2 + 1
               xml_hour2 = xml_hour

            xml_temp = 0
            for temp in forecast.findall('feelslike'):
               xml_tempp = int(temp.find('metric').text)
               break
            xml_pop = int(forecast.find('pop').text)
      
            labels.append(xml_hour)
            hours.append(xml_hour + (xml_day2*24))
            temp_feelslike.append(xml_tempp)
            rainchance.append(xml_pop)

            i = i + 1
            if i >= (nb_temperature):
               break;
      if len(labels) == nb_temperature and len(hours) == nb_temperature and len(temp_feelslike) == nb_temperature and len(rainchance) == nb_temperature:
         old_temperature = result
      else:
         result = ""
   if len(result) < 1 :
      result = old_temperature
   if len(result) > 0 :
      temperature(result, ctx)
##############################################################
   log.write("Refreshing meteo...OK!")

def temperature(result, ctx):
   labels = []
   hours = []
   temp_feelslike = []
   rainchance = []
   xml_hour2 = 0
   xml_day2 = 0
   i = 0

   locale.setlocale(locale.LC_ALL, 'fr_FR.utf8')

   xml = ElementTree.fromstring(result)
   for root in xml.findall('hourly_forecast'):
      for forecast in root.findall('forecast'):
         for fcttime in forecast.findall('FCTTIME'):
            xml_hour = int(fcttime.find('hour_padded').text)
            if xml_hour2 > xml_hour:
               xml_day2 = xml_day2 + 1
            xml_hour2 = xml_hour

         xml_tempp = 0
         for temp in forecast.findall('feelslike'):
            xml_tempp = int(temp.find('metric').text)
            break
         xml_pop = int(forecast.find('pop').text)
      
         labels.append(xml_hour)
         hours.append(xml_hour + (xml_day2*24))
         temp_feelslike.append(xml_tempp)
         rainchance.append(xml_pop)

         i = i + 1
         if i >= (nb_temperature):
            break;
#Lissage
   lisse = interp1d(hours, temp_feelslike, kind='cubic')	
   hours_lisse = np.linspace(min(hours), max(hours), 40)
   temp_lisse = lisse(hours_lisse)

   matplotlib.rc('axes',edgecolor='white')
   font = {'family' : 'Droid Sans', 'size': 8}
   matplotlib.rc('font', **font)
   fig1 = plt.figure(figsize=(4.5, 1.5),dpi=100)
   
#Temperature   
   ax1 = fig1.add_subplot(111)
   ax1.xaxis.label.set_color('white')
   ax1.spines['top'].set_visible(False)
   ax1.spines['right'].set_visible(False)
   ax1.get_xaxis().tick_bottom()
   ax1.get_yaxis().tick_left()
   ax1.tick_params(axis='x', colors='white')
   ax1.tick_params(axis='y', colors='white')

   plt.plot(hours_lisse,temp_lisse,'w-',lw=2)
   plt.gcf().subplots_adjust(right=0.75)
   plt.axis([min(hours),max(hours),min(temp_lisse)-0.5,max(temp_lisse)+0.5])
   ax1.set_ylabel('Temp.', color='w')
#Temperature Labels
   plt.xticks(hours);
   for x in range (0,len(labels)):
      if x % 2 == 1:
         labels[x] = ""
      else:
         labels[x] = str(labels[x]) + "h"
   ax1.set_xticklabels(labels)
   ax1.set_ylabel('Temp.', color='w')
#Probabilité of Pluie (POP)
   ax2 = ax1.twinx()
   ax2.spines['top'].set_visible(False)
   ax2.spines['left'].set_visible(False)
   ax2.set_ylabel('Pluie', color='w')
   ax2.tick_params(axis='y', colors='white')
   ax2.axis([min(hours),max(hours),0,100])
   ax2.bar(hours,rainchance, 1, color='b', alpha=0.5)
   ax2.set_ylabel('Pluie', color='w')
#Sauvegarde du graphique
   png = BytesIO()
   fig1.savefig(png, format='png',transparent=True)
   plt.close('all')
   png.seek(0) 
   temp_surface = cairo.ImageSurface.create_from_png(png);
   ctx.set_source_surface(temp_surface, 0, 180)
   ctx.paint()
   png.close()
   temp_surface.finish()

def today(result,ctx,pop):
   locale.setlocale(locale.LC_ALL, 'fr_FR.utf8')

   xml = ElementTree.fromstring(result)
   temp_c = xml.findtext(".//temp_c")
   feelslike_c = xml.findtext(".//feelslike_c")
   icon = xml.findtext(".//icon")
   relative_humidity = xml.findtext(".//relative_humidity")
   wind_kph = xml.findtext(".//wind_kph")
   epoch = xml.findtext(".//observation_epoch")

   ctx.set_source_rgba(1, 1, 1, 1)

   #Nom de la ville
   ctx.set_font_size(25)
   ctx.move_to(15,35)
   ctx.text_path("Strasbourg")
   ctx.fill()

   #Heure de la prévision today
   ctx.set_font_size(14)
   ctx.move_to(15,60)
   ctx.text_path(strftime("%A %H:%M", localtime(int(epoch))).title())
   ctx.fill()

   #degrees
   ctx.set_font_size(55)
   ctx.move_to(120,125)
   ctx.text_path(str(int(float(temp_c))) + '°')
   ctx.fill()
   ctx.set_font_size(16)
   ctx.move_to(15,165)
   ctx.text_path("Ressenti comme " + str(int(float(feelslike_c))) + '°' + 'C')
   ctx.fill()

   #Precipitation
   ctx.set_font_size(16)
   ctx.move_to(222,90)
   ctx.text_path("Précipitations : " + pop + "%")
   ctx.fill()

   #Humidite
   ctx.set_font_size(16)
   ctx.move_to(222,115)
   ctx.text_path("Humidité : " + relative_humidity)
   ctx.fill()

   #vent
   ctx.set_font_size(16)
   ctx.move_to(222,140)
   ctx.text_path("Vent : " + wind_kph + "km/h")
   ctx.fill()

#image
   if icon in icons:
      ctx.set_source_surface(icons[icon], 35, 75);
   else:
      ctx.set_source_surface(icons["unknown"], 35, 75);
   ctx.paint();

def draw(ctx, x, y):
   ctx.set_source_surface(meteo_surface, x, y );
   ctx.paint()

def previsions(result, ctx):
   locale.setlocale(locale.LC_ALL, 'fr_FR.utf8')

   xml = ElementTree.fromstring(result)
   i = 0
   fcast = []
   pop = 0
   for root in xml.findall('forecast'):
      for txt in root.findall('simpleforecast'):
         for days in txt.findall('forecastdays'):
            for day in days.findall('forecastday'):
               for date in day.findall('date'):
                  epoch_tmp = date.find('epoch').text
               period_tmp = day.find('period').text
               for highs in day.findall('high'):
                  high = highs.find('celsius').text
               for lows in day.findall('low'):
                  low = lows.find('celsius').text
               pop_tmp = day.find('pop').text
               icon_tmp = day.find('icon').text
               if period_tmp == str(1):
                  pop = pop_tmp
               fcast.append([epoch_tmp, high, low, pop, icon_tmp])
               i = i + 1
               if i >= nb_fcast:
                  break;
   
   ctx.set_source_rgba(1, 1, 1, 1)

   #Previsions
   #saute le premier
   fcast.pop(0)
   offset=45

   for day in fcast:
#Nom du jour
      ctx.set_font_size(14)
      text = strftime("%A", localtime(int(day[0]))).title()
      ctx.move_to(offset+middle(ctx,text,14,40),357)
      ctx.text_path(text)
#Température
      text = str(int(float(day[2]))) + "° / " + str(int(float(day[1]))) + "°"
      ctx.move_to(offset+middle(ctx,text,14,40),420)
      ctx.text_path(text)
      ctx.fill()

      offset = offset + 75

   offset=45
   for day in fcast:
#Icone
      if day[4] in icons:
         ctx.set_source_surface(icons[day[4]],offset+((40-icons[day[4]].get_width())/2), 365);
      else:
         ctx.set_source_surface(icons["unknown"],offset+((40-icons["unknown"].get_width())/2), 365);
      ctx.paint();

      offset = offset + 75

   return pop

def init(root_path):
   global meteo_surface
   i = 0
   meteo_surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
   for dirname, dirnames, filenames in os.walk(root_path + "/meteo"):
      for filename in filenames:
         image, extension = os.path.splitext(filename)
         if extension.upper() == '.PNG' :
            icons[image] = cairo.ImageSurface.create_from_png(dirname + '/' + filename)            
            i = i + 1
   log.cat(str(i) + " logos",end="")
